#! /usr/bin/perl

my @liststack = ();
my @indent = (3,6,9,12);
my $i =0 ;
my $spacing;
my $type = "";
my $inverbatim = 0;

while (<>) {

  # create verbatim environments
  s/^<verbatim>/{\\small \\begin{verbatim}/;    
  s/^<\/verbatim>/\\end{verbatim} }/;
  s/^<pre>/{\\small \\begin{verbatim}/;
  s/^<\/pre>/\\end{verbatim} }/;

  if ( /begin{verbatim}/.../end{verbatim}/ ) {
    print;
    next;
  }

  if ( /%LATEX%/.../%ENDLATEX%/ ) {
    print;
    next;
  }

  # delete comments
  next if /^\<\!\-\-$/../\-\-\>$/;

  # translate =xxx= into verbatim statements:
  #   inline, end of line, start of line
  s/([ 	\(])=(.*?)=([	 ,.\?\)])/\1\\verb!\2!\3/g;
  s/^(\s*)=(.*?)=:/\1\\verb!\2!:/;
  s/([	 \(])=(.*?)=([	 ,.\)\?]?)$/\1\\verb!\2!\3/g;
  s/^=(.*?)=([	 ,.\?])/ \\verb!\1!\2/g;
  s/^=(.*?)=([	 ,.\?]?)$/ \\verb!\1!\2/g;

  # escape all \ first so it doesn't double the inserted latex commands
  # don't do it if inside a verb.
  if ( ! /\\verb\![^!]*\\/ ) {
    s/\\/\\\\/g;
    # make sure to undo the doubiling on the verbs.
    s/\\\\verb!/\\verb!/g;
    # make sure to undo the doubling on the embedded index commands.
    s/\\\\index{/\\index{/g;
  }

  # replace line break
  s/\<br [ \/]*\>/\\\\/;

  # handle blockquotes.
  s/^\<blockquote\>/\\begin{quote}/;
  s/^\<\/blockquote\>/\\end{quote}/;

  # escape _  (math mode character in latex) unless in verbatim
  if ( ! /\\verb\![^!]*[\_]/ ) {
    s/_/\_/g;
  }

  # output [[url][text]] as \2 (\url{\1})
  s/\[\[(http:\/\/[^]]*)\]\[([^]]*)\]\]/\2 (\\url{\1})/;

  # output url's as \url{...} capture trailing punctuation.
  s/([^!{])(http:\/\/[^!\s]*)([.,][^\w])/\1\\url{\2}\3/;

  # output url's as \url{...} capture trailing punctuation if end of sentence
  # but not if embedded in url.
  s/^(http:\/\/[^!\s]*)([.,]\W)/\\url{\1}\2/;

  # output url's as \url{...} if no trailing puncutation.
  # if captured by above the must not match { will stop this from matching.
  s/([^!{])(http:\/\/[^!\s]*)([.,]?)/\1\\url{\2}\3/;
  # output url's as \url{...}
  s/^(http:\/\/[^!\s]*)([.,]?)/\\url{\1}/;

  # make references work kind of.
  # translate [[xxx#yyy][zzzz]] into:
  # zzzz section \ref{yyy}
  s/\[\[[A-z]+\#([A-z]*)\]\[([^]]*)\]\]/\2 section~\\ref{\1}/g;

# make references work kind of.
# translate [[#yyy][zzzz]] into:
# zzzz section \ref{yyy}
s/\[\[\#([A-z]*)\]\[([^]]*)\]\]/\2 section~\\ref{\1}/g;

# make references work kind of.
# translate WikiWord#WikiWordTwo into:
# section \ref{WikiWordTwo}
s/[A-Z][A-z]*\#([A-z]*)/section~\\ref{\1}/g;

    # start itemized
    # detect indented line at current level
    $i = $indent[@liststack];
    if (/^ {$i}[*] / && (@liststack*3 != $i) ) {
      print " "x(@liststack*3) . "\\begin{itemize}\n";
      push(@liststack, "itemize");
    }

    # outdent
    if (@liststack) { # check to see if we need to outdent
      $i = @liststack * 3;
      if (! /^ {$i}/ ) {
	$type = pop(@liststack);
	print " "x(@liststack*3) . "\\end{$type}\n";
      }
      $i = @liststack * 3;
      # pop a second level if needed
      if (! /^ {$i}/ ) {
	$type = pop(@liststack);
	print " "x(@liststack*3) . "\\end{$type}\n";
      }
      $i = @liststack * 3;
      # pop a third level because I am too lazy to write a loop.
      if (! /^ {$i}/ ) {
	$type = pop(@liststack);
	print " "x(@liststack*3) . "\\end{$type}\n";
      }
    }

    # itemize items
    if (@liststack) {
      $i = @liststack * 3;
      $spacing = " "x$i;
      s/^ {$i}[*] /$spacing\\item /;
    }
    
    # start enumerate
    # detect indented line at current level
    $i = $indent[@liststack];
    if (/^ {$i}[0-9]+/ && (@liststack*3 != $i) ) {
      print " "x(@liststack*3) . "\\begin{enumerate}\n";
      push(@liststack, "enumerate");
    }
    
    if (@liststack) {
      $i = @liststack * 3;
      $spacing = " "x$i;
      s/^ {$i}[0-9]+/$spacing\\item/;
    }

    # start description
    # detect indented line at current level
    # allow three space seperated words in the label.
    $i = $indent[@liststack];
    if (/^ {$i}([^\s]+?(\s+[^\s:]*){0,2}):\s/ && (@liststack*3 != $i) ) {
      print " "x(@liststack*3) . "\\begin{description}\n";
      push(@liststack, "description");
    }
    
    if (@liststack) {
      $i = @liststack * 3;
      $spacing = " "x$i;
      s/^ {$i}([^\s]+?(\s+[^\s:]*){0,2}):\s/$spacing\\item\[\1\] /;
    }

# replace paragraphs with blank lines. Do list environments first
# as <p /> in twiki is way to do multi para itemized items.
s/\<p \/\>//;

# replace all html entities
s/\&gt\;/\\verb!>!/g;
s/\&lt\;/\\verb!<!/g;

# translate section heads
s/^\-\-\-\+\+\+\+\+\+ (.*)/\\subparagraph{\1}/;
s/^\-\-\-\+\+\+\+\+ (.*)/\\paragraph{\1}/;
s/^\-\-\-\+\+\+\+ (.*)/\\subsubsection{\1}/;
s/^\-\-\-\+\+\+ (.*)/\\subsection{\1}/;
s/^\-\-\-\+\+ (.*)/\\section{\1}/;
s/^\-\-\-\+ (.*)/\\chapter{\1}/;

# translate *xxx* into emphasis
s/(\W)\*([^*]*)\*(\W)/\1\{\\em \2\}\3/;
s/^\*([^*]*)\*[	 ]/{\\em \1} /;
s/[	 ]\*([^*]*)\*$/ {\\em \1}/;

# manage inline labels
s/^#([A-Z][a-z]*[A-z]*)/\\label{\1}/;

# strip twiki variable
s/\%[A-Z]*(\{\})?\%//g;

# escape all % (comments in latex) if not in verb
if ( ! /\\verb\![^!]*%/ ) {
  s/%/\\%/g;
}

  # escape all < > unless inside verbatim
  if ( ! /\\verb\![^!]*[<>]/ ) {
    s/</\\verb!<!/g;
    s/>/\\verb!>!/g;
  }

# escape all $^_ (math mode characters in latex) unless in verbatim
if ( ! /\\verb\![^!]*[\$\_\^\&]/ ) {
  s/([\$\^\&])/\\\1/g; s/_/\\_/g;
}

# escape all # (macro arg in latex) not in verb
  if ( ! /\\verb\![^!]*\#/ ) {
    s/\#/\\#/g;
  }

print;
}
