#comment This file has to work under any OS. It currently works under
#comment both solaris and linux. The tools 
#comment that diverge in functionality are:
#comment
#comment test - the /bin/sh builtin doesn't have the -e flag on
#comment         solaris. However -r returns true when file is mode
#comment         000 and test is run by root so you can replace -e
#comment         with -r and have it work in the cases we care
#comment         about. Tested for foo being a file, dangling link,
#comment         resolved link and directory. Matches -e output.
#comment xargs - -r flag missing to prevent command invocation if
#comment         stdin to xargs is empty. Looks like a bare rm -f
#comment         doesn't error under solaris, so we can do without it
#comment mv - uses --backup functionality
#comment
#comment to get around this problem, put /etc/config/bin first in
#comment path.  link gnu tools under solaris at /opt/sfw/bin, or write
#comment wrappers to massage command args.

#comment****h* Config/rdist_macros
#comment* NAME
#comment*    rdist_macros -- macros that can be used to generate rdist commands
#comment* SYNOPSIS
#comment*    filepp -I <directory> <file>
#comment*
#comment*    #include "rdist_macros.fpp" (at top of file to be processed)
#comment* PURPOSE
#comment* A number of different operations can be defined in rdist distfiles.
#comment* However they are often composed of multiple simple rdist commands.
#comment* This module defines a number of macros for use when generating
#comment* a distfile for Config. It also supports adding a tag that can
#comment* be used to filter out the housekeeping commands making it
#comment* easier to see what actions have occurred.
#comment******

  #comment allow use of #bigdef
#pragma filepp UseModule bigdef.pm
  #comment allow use of #bigfunc
#pragma filepp UseModule bigfunc.pm
  #comment allow use of substr() and other formatting commands
#pragma filepp UseModule format.pm
  #comment allow extending of definitions
#pragma filepp UseModule defplus.pm
  #comment define predefplus for generating new classes using set operations
#pragma filepp UseModule predefplus.pm
  #comment define predefine for implementing set operations
#pragma filepp UseModule predefine.pm
  #comment define foreach to allow definition of LABELBLOCK
#pragma filepp UseModule foreach.pm
  # space seperated lists by default
#foreachdelim /\s+/
  #comment allow appending to a macro from the *value* of another macro
#pragma filepp UseModule predefplus.pm
  #comment allow definition a macro from the *value* of another macro
#pragma filepp UseModule predefplus.pm

  #comment****im* rdist_macros/RDIST_SPECIAL_FILTER
  #comment* NAME
  #comment*   RDIST_SPECIAL_FILTER - use with special commands to filter it from the output
  #comment* SYNOPSIS
  #comment*   cmdspecial "/bin/ls RDIST_SPECIAL_FILTER";
  #comment* PURPOSE
  #comment* This macro is used to tag special/cmdspecial command lines
  #comment* so that they can be filtered out of the rdist output later
  #comment* cleaning up the reporting.
  #comment* EXAMPLE
  #comment* With this stanza:
  #comment*    RUN($C/Rdist, ${FEDORA_HOSTS} + ${CENTOS_HOSTS});
  #comment*        cmdspecial "grep tw_cli /etc/sudoers; exit 0 RDIST_SPECIAL_FILTER";
  #comment* The output of rdist can be run through a grep -v for the
  #comment* expansion of RDIST_SPECIAL_FILTER. This is used internally to the
  #comment* rdist_macro package for the most part and is used to allow
  #comment* filtering of the housekeeping commands generated by BACKUP
  #comment* and other macros.
  #comment******

#define RDIST_SPECIAL_FILTER  ;#filter
  #comment****m* rdist_macros/BACKUP
  #comment* NAME
  #comment*   BACKUP - manage backup copies of pushed files
  #comment* SYNOPSIS
  #comment*   BACKUP(5);
  #comment* PURPOSE
  #comment* The following macro used in the command block of an rdist stanza
  #comment* keeps a number of backups of all files pushed by
  #comment* a stanza. This can be used in place of SAVEINSTALL in cases
  #comment* where you need to use the rdist intall command instead. Note that
  #comment* this will work only if -osavebackups is defined on install
  #comment* command to make the original backup copy.
  #comment*
  #comment* The number of backups to keep is specified as the only
  #comment* argument.
  #comment*
  #comment* Backup files are saved as:
  #comment*    * target_file_name.SAVED -- the most recent backup
  #comment*    * target_file_name.SAVED.~N~ -- prior backups. The
  #comment*      smaller the number, the older the backup.
  #comment*
  #comment* EXAMPLE
  #comment* label-manual:
  #comment* $C/ntp/clients.conf -> ${NTP_SLAVES}
  #comment*	install -osavetargets /etc/ntp.conf;
  #comment* 	cmdspecial "/etc/rc.d/init.d/ntpd restart" ;
  #comment* 	BACKUP(10);
  #comment* SEE ALSO
  #comment*   SAVEINSTALL
  #comment******

#bigdef  BACKUP(MAX_VERSIONS)
special "PATH=/etc/config/bin:$PATH; export PATH; if [ -r ${REMFILE}.OLD ]; then mv --backup=numbered ${REMFILE}.OLD ${REMFILE}.SAVED; ls ${REMFILE}.SAVED.*~ 2>/dev/null | sort -t '~' -k 2,2nr | sed '1,##MAX_VERSIONS##d' | xargs -r rm -f ; fi RDIST_SPECIAL_FILTER"##
#endbigdef

  #comment****m* rdist_macros/BACKUPTO
  #comment* NAME
  #comment*   BACKUPTO - backup copies of pushed files to a new directory
  #comment* SYNOPSIS
  #comment*   BACKUPTO(/var/rdist/backup/qmail/controls,5);
  #comment* PURPOSE
  #comment* The following macro used in the command block of an rdist
  #comment* stanza keeps a number of backups of all files pushed by a
  #comment* stanza. Unlike BACKUP this saves the files to a new
  #comment* directory.  Note that this will work only if -osavebackups
  #comment* is defined on install command to make the original backup
  #comment* copy.
  #comment*
  #comment* The the target directory and number of backups to keep are
  #comment* given to the macro. The target directory will be created
  #comment* mode 700 if 
  #comment* needed. The target directory must be an absolute path
  #comment* starting with a /. This can be combined with the
  #comment* SAVEINSTALL macro, but it duplicates the functionality of
  #comment* the BACKUP macro and should be used in place of it.
  #comment*
  #comment* This macro is particularly useful if you are installing an
  #comment* entire directory with the remove option as the backup
  #comment* copies will be removed on each update. Also for directories
  #comment* such as logrotate that don't silently ignore the backup
  #comment* files, this macro allows you to save the backups
  #comment* elsewhere.  Backup files are saved as: *
  #comment* target_file_name.SAVED -- the most recent backup *
  #comment* target_file_name.SAVED.~N~ -- prior backups. The smaller
  #comment* the number, the older the backup.
  #comment*
  #comment* This will also work with the SAVEINSTALL macro.
  #comment*
  #comment* EXAMPLE
  #comment* This moves the backup file to /etc/ntp.backup:
  #comment*   label-manual:
  #comment*   $C/ntp -> ${NTP_SLAVES}
  #comment*	  install -osavetargets,remove /etc/ntp;
  #comment* 	  cmdspecial "/etc/rc.d/init.d/ntpd restart" ;
  #comment* 	  BACKUPTO(/etc/ntp.backup, 10);
  #comment*
  #comment* This moves the backup file to /etc/cron.d.backup so that
  #comment* the old file isn't executed by cron:
  #comment*   label-manual:
  #comment*   $C/cron.d/postgres -> ${POSTGRES_HOSTS}
  #comment*	  SAVEINSTALL(/etc/cron.d, 3);
  #comment* 	  BACKUPTO(/etc/cron.d.backup, 10);
  #comment*
  #comment* Note in this case, the count for the SAVEINSTALL is ignored and 
  #comment* the count provided to BACKUPTO is the one that is used.
  #comment* SEE ALSO
  #comment*   SAVEINSTALL, BACKUP
  #comment******

#bigdef  BACKUPTO(TARGETDIR, MAX_VERSIONS)
special "PATH=/etc/config/bin:$PATH; export PATH; U=`umask`; umask 077; [ ! -d TARGETDIR ] && mkdir -p TARGETDIR; umask $U; cd TARGETDIR; if [ -r ${REMFILE}.OLD ]; then mv --backup=numbered ${REMFILE}.OLD ${BASEFILE}.SAVED; fi; if [ -r ${REMFILE}.SAVED ]; then mv --backup=numbered ${REMFILE}.SAVED ${BASEFILE}.SAVED; fi; ls ${BASEFILE}.SAVED.*~ 2>/dev/null | sort -t '~' -k 2,2nr | sed '1,##MAX_VERSIONS##d' | xargs -r rm -f  RDIST_SPECIAL_FILTER"##
#endbigdef

  #comment****im* rdist_macros/DATESTAMP
  #comment* NAME
  #comment*   DATESTAMP - a current datestamp
  #comment* SYNOPSIS
  #comment*   DATESTAMP
  #comment* PURPOSE
  #comment* Provides the date of execution as: YYYY-MM-DD-HH-mm-SS
  #comment* where MM is the 2 digit month and mm is a two digit minute.
  #comment* USED BY
  #comment* FORCERUN, RUN
  #comment******

  #comment The __TIME__ macro is broken up because the
  #comment format is 23:00:23 and the :'s give rdist heartburn since the
  #comment : is used to seperate files in the FILES variable for
  #comment cmdspecial commands. The REV macro is the revision of the
  #comment SVN repository and is passed in on the filepp command line.

#define DATESTAMP __ISO_DATE__-substr(__TIME__,0,2)-substr(__TIME__,3,2)-substr(__TIME__,6,2)

  #comment****m* rdist_macros/FORCERUN
  #comment* NAME
  #comment*   FORCERUN - always run special commands
  #comment* SYNOPSIS
  #comment*   FORCERUN;
  #comment* PURPOSE
  #comment* The following macro defines an install rule that always
  #comment* installs a file and runs the associated special/cmdspecial
  #comment* commands. Note that it should be used only with a single
  #comment* file.
  #comment* EXAMPLE
  #comment* This example will always run /etc/rc.d/init.d/ntpd restart
  #comment* when executed.
  #comment*   label-manual:
  #comment*   $C/ntp/clients.conf -> ${NTP_SLAVES}
  #comment*	FORCERUN;
  #comment*	cmdspecial "/etc/rc.d/init.d/ntpd restart" ;
  #comment* NOTES
  #comment* This may leave files of the form
  #comment* /tmp/rdist-2006-05-24-12-03-23-766 around. They can be deleted.
  #comment* Also this should not be used to push a file that has
  #comment* sensitive data in it as the contents of the file may be
  #comment* readable on the remote system.
  #comment* SEE ALSO
  #comment*   RUN
  #comment* USES
  #comment* DATESTAMP
  #comment* USED BY
  #comment* RUN
  #comment******
#bigdef  FORCERUN install -oquiet /tmp/rdist.DATESTAMP.REV ;
	cmdspecial "rm -fr /tmp/rdist.DATESTAMP.REV RDIST_SPECIAL_FILTER"##
#endbigdef

  #comment FIXME good idea, but requires multiple pass by filepp which it
  #comment doesn't do. ENDLABELBLOCK not substituted when foreach loop
  #comment inside LABELBLOCK macro is evaluated it is looking for the
  #comment #endforeach which isn't there because ENDLABELBLOCK isn't
  #comment substituted yet. Need to create labelblock.pm or some such.
  #comment ****m* rdist_macros/LABELBLOCK
  #comment * NAME
  #comment *   LABELBLOCK - repeat a rdist commands with different labels
  #comment * SYNOPSIS
  #comment *   LABELBLOCK(label1, label2,label3)
  #comment *   ENDLABELBLOCK
  #comment * PURPOSE
  #comment * This command repeats the rdist commands between LABELBLOCK
  #comment * and ENDLABELBLOCK using a different label for each
  #comment * repititon. This is useful for creating a specific label
  #comment * for the rdist action as well as a more general label
  #comment * allowing the execution of just the labeled block.
  #comment * EXAMPLE
  #comment * To define the labels/targets ldap and ldap.cert use:
  #comment *   LABELBLOCK(ldap, ldap.cert);
  #comment *   $C/ldap/openldap/cacert.pem -> ${LDAP_SLAVES}
  #comment *            VERIFYFILE(/etc/openldap/cacert.pem);
  #comment *   ENDLABELBLOCK;
  #comment ******
#comment #bigfunc LABELBLOCK(ARGS...)
#comment #foreach LABEL ARGS
#comment #LABEL##:
#comment ##endbigfunc

#comment #define ENDLABELBLOCK #endforeach

  #comment****m* rdist_macros/IFPOSTACTION
  #comment* NAME
  #comment*   IFPOSTACTION - check to see if there are postinstall commands to run 
  #comment* SYNOPSIS
  #comment*   cmdspecial "IFPOSTACTION(ACTION,[OPTION]); shell command";
  #comment* PURPOSE
  #comment* This macro will prevent "shell command" from
  #comment* executing unless a file was pushed that uses the
  #comment* POSTINSTALL macro with the same ACTION and OPTION
  #comment* arguments.
  #comment*
  #comment* ACTION and OPTION are single words consisting of letters,
  #comment* digits and the underscore '_'.
  #comment* 
  #comment* When called with the optional OPTION argument,
  #comment* IFPOSTACTION can be called in multiple cmdspecial
  #comment* commands and across multiple rdist stanzas. All it does
  #comment* is query if the particular ACTION/OPTION pair has been
  #comment* asserted by a prior POSTACTION macro.
  #comment*
  #comment* If the OPTION argument is not present, it must be the last
  #comment* or only cmdspecial call as it removes/resets the action
  #comment* and future IFPOSTACTION for the same action will believe
  #comment* that there is no action to be done. If you want to test
  #comment* for the existence of a pending ACTION for any OPTION, use
  #comment* the option consisting of a colon ':'.
  #comment*
  #comment* Don't use " or ' marks around or in the OPTION argument.
  #comment*
  #comment* EXAMPLE
  #comment* This example will run /etc/init.d/sshd stop if a prior
  #comment* installed file included POSTINSTALL(sshd, stop), it will
  #comment* also run rekey if POSTINSTALL(sshd, rekey) was used, and
  #comment* will run start if any file was installed that called 
  #comment* POSTINSTALL(sshd), or POSTINSTALL(sshd, ...):
  #comment* 
  #comment*   POSTINSTALL-ACTIONS:
  #comment*   $C/Distfile -> ${SSHD_SLAVES}
  #comment*	FORCERUN;
  #comment*	cmdspecial "IFPOSTACTION(sshd, stop); /etc/init.d/ssh stop" ;
  #comment*	cmdspecial "IFPOSTACTION(sshd, rekey); /etc/init.d/ssh rekey" ;
  #comment*	cmdspecial "IFPOSTACTION(sshd); /etc/init.d/sshd start" ;
  #comment*
  #comment* In the example below, "/etc/init.d/ssh start" will never
  #comment* be called because it occurs after a call to
  #comment* IFPOSTACTION(sshd) which deletes the file that
  #comment* IFPOSTACTION(sshd, start) needs to determine if it should
  #comment* run.
  #comment*
  #comment* EXAMPLE
  #comment*   POSTINSTALL-ACTIONS:
  #comment*   $C/Distfile -> ${SSHD_SLAVES}
  #comment*	FORCERUN;
  #comment*	cmdspecial "IFPOSTACTION(sshd, stop); echo stopping; /etc/init.d/ssh stop" ;
  #comment*	cmdspecial "IFPOSTACTION(sshd);" ;
  #comment*	cmdspecial "IFPOSTACTION(sshd, start); /etc/init.d/ssh start" ;
  #comment*
  #comment* NOTES
  #comment* Rdist adds the POSTINSTALL-ACTIONS by default to the list of
  #comment* targets to be executed.
  #comment*
  #comment* Using the IFPOSTACTION macro suppresses the printing of
  #comment* the entire cmdspecial or special except at --verbose level
  #comment* 5 in Rdist. If the command is executed, the
  #comment* macro puts out "executing ACTION:OPTION". You can
  #comment* suppliment it with your own echo command as part of the
  #comment* special/cmdspecial.
  #comment* 
  #comment* The POSTINSTALL-ACTIONS must come after all targets in the
  #comment* distfile that could possibly set the ACTION flag. It is
  #comment* best if the POSTINSTALL-ACTIONS stanza is last in the
  #comment* distfile.
  #comment*
  #comment* This is a different type of macro in that is it used
  #comment* inside a cmdspecial, so it must be used in a quoted string.
  #comment* 
  #comment* SEE ALSO
  #comment*   POSTACTION
  #comment*   FORCERUN
  #comment******
#define  IFPOSTACTION(ACTION,OPTION...) ACTFILE=/etc/config/postinstall/ACTION; if test -r $ACTFILE; then if test -n \"OPTION\"; then if ! grep \"^:OPTION\" $ACTFILE > /dev/null; then exit 0; fi; else rm -f $ACTFILE; fi; else exit 0; fi; echo executing ACTION:OPTION; true TrIgGeR

  #comment****m* rdist_macros/NOOP
  #comment* NAME
  #comment*   NOOP - Allow a rule in distfile that will never be executed
  #comment* SYNOPSIS
  #comment*   NOOP;
  #comment* PURPOSE
  #comment* It is useful to have dummy targets in the distfile to allow
  #comment* Rdist to updated directories that are not pushed via rdist.
  #comment* This creates a rule to push $C/Config/bin/Rdist to a non-existent
  #comment* target host. It takes the same role as the : command in shell.
  #comment* EXAMPLE
  #comment* The following will never actually push any files, but will allow
  #comment* Rdist to update the netdev directory:
  #comment*   netdev:
  #comment*   NOOP;
  #comment******

#bigdef  NOOP $C/Config/bin/Rdist -> DuMmYHoSt.NOOP
	install /tmp/RDIST##
#endbigdef

  #comment****m* rdist_macros/POSTACTION
  #comment* NAME
  #comment*   POSTACTION - register an action for execution after all updates occur
  #comment* SYNOPSIS
  #comment*   POSTACTION(ACTION,[OPTION])
  #comment* PURPOSE
  #comment* 
  #comment* rdist supports actions that are run when a file is
  #comment* updated, however if you have multiple files that need to
  #comment* run the same action, and the files are in different rdist
  #comment* stanza's, you currently have to execute the action for
  #comment* every file.
  #comment*
  #comment* POSTACTION allows an action to be triggered only once
  #comment* regardless of how many files are installed. E.G. you
  #comment* need to restart sshd when the ssh keys change and when the
  #comment* /etc/init.d/sshd file is updated. Because the same rdist
  #comment* stanza/install command can't be used to push both files,
  #comment* you would end up restarting sshd twice if both files are
  #comment* updated. However using POSTACTION sshd can be restarted
  #comment* only once if either or both files are updated.
  #comment*
  #comment* POSTACTION is used to register the need for an action. The
  #comment* action is defined and executed using IFPOSTACTION.
  #comment*
  #comment* EXAMPLE
  #comment* The following installs ssh keys and a new sshd startup
  #comment* file and registers the need to restart sshd. Then a
  #comment* POSTINSTALL-ACTIONS performs the actual action.
  #comment*   ssh:
  #comment*   $C/sshd/rc/sshd -> ${SSHD_SLAVES}
  #comment*	install -osavetargets /etc/init.d/sshd;
  #comment* 	POSTACTION(sshd-restart) ;
  #comment*   ssh:
  #comment*   $C/sshd/keys/a_host/*id_rsa* -> ${SSHD_SLAVES}
  #comment*	install -osavetargets /etc/ssh/.;
  #comment* 	POSTACTION(sshd-restart,keys) ;
  #comment* 	POSTACTION(sshd-restart,anotheroption) ;
  #comment*   ... at the very end of the Distfile
  #comment*   POSTINSTALL-ACTIONS:
  #comment*   $C/Distfile -> ${SSHD_SLAVES}
  #comment*	FORCERUN;
  #comment* 	cmdspecial "IFPOSTACTION(sshd-restart,keys); touch /etc/newkeys";
  #comment* 	cmdspecial "IFPOSTACTION(sshd-restart); /etc/init.d/sshd restart";
  #comment* Note that "touch /etc/newkeys" won't be done
  #comment* if the only change was to the /etc/init.d/sshd file as the
  #comment* "keys" option won't be registered.
  #comment* NOTE
  #comment* Don't use " or ' marks in the OPTION argument.
  #comment* SEE ALSO
  #comment*   IFPOSTACTION
  #comment******

#define POSTACTION(ACTION, OPTION...) cmdspecial "echo \":OPTION\" >> /etc/config/postinstall/ACTION RDIST_SPECIAL_FILTER"

  #comment****m* rdist_macros/REPORTTARGET
  #comment* NAME
  #comment*   REPORTTARGET - report the name of the updated file as it is known on the target machine.
  #comment* SYNOPSIS
  #comment*   REPORTTARGET;
  #comment* PURPOSE
  #comment* The following macro defines a command that reports the
  #comment* names of the updated files using the pathname on the
  #comment* target host.
  #comment* EXAMPLE
  #comment* The following example will report
  #comment* "*** /etc/ntp.conf updated." when it is updated.
  #comment* as opposed to the standard "Updating /config/ntp/clients.conf"
  #comment* reported by rdist (assuming $C is /config).
  #comment*
  #comment*   label-manual:
  #comment*   $C/ntp/clients.conf -> ${NTP_SLAVES}
  #comment*	install -osavetargets /etc/ntp.conf;
  #comment* 	cmdspecial "/etc/rc.d/init.d/ntpd restart" ;
  #comment*	REPORTTARGET;
  #comment******

#define REPORTTARGET special "echo \"***	${REMFILE} updated.\" RDIST_SPECIAL_FILTER"

  #comment****m* rdist_macros/RUN
  #comment* NAME
  #comment*   RUN - always run the provided commands
  #comment* SYNOPSIS
  #comment*   RUN($C/Config/distfile/settings, ${FEDORA_HOSTS} + ${CENTOS_HOSTS});
  #comment*    		cmdspecial "cd /etc/config; ./HOSTVERIFY" ;
  #comment* PURPOSE
  #comment* This is an abbreviated version of the FORCERUN command.
  #comment* It's two arguments set up a rule to always run the
  #comment* following special/cmdspecial commands on the remote host(s).
  #comment* It can be used only with a single file.
  #comment*
  #comment* Note the ';' after the RUN command.
  #comment* EXAMPLE
  #comment* The following example will report
  #comment* run "cd /etc/config; ./HOSTVERIFY" every time it is invoked.
  #comment*   label-manual:
  #comment*   RUN($C/Config/distfile/settings, ${FEDORA_HOSTS} + ${CENTOS_HOSTS});
  #comment*    		cmdspecial "cd /etc/config; ./HOSTVERIFY" ;
  #comment*
  #comment* It generates something like:
  #comment*   label-manual:
  #comment*   $C/Config/distfile/settings ->${FEDORA_HOSTS} + ${CENTOS_HOSTS})
  #comment*  		install -oquiet /tmp/rdist.2006-05-24-13-33-02.798 ;
  #comment*    		cmdspecial "cd /etc/config; ./HOSTVERIFY" ;
  #comment* SEE ALSO
  #comment* FORCERUN
  #comment* USES
  #comment* FORCERUN, DATESTAMP
  #comment******
#bigdef  RUN(FILE, HOSTS) FILE -> HOSTS
	FORCERUN##
#endbigdef

  #comment****m* rdist_macros/SAVEINSTALL
  #comment* NAME
  #comment*   SAVEINSTALL - Install a file saving some number of copies.
  #comment* SYNOPSIS
  #comment*   SAVEINSTALL(file/directory, count, install_args);
  #comment* 
  #comment*   SAVEINSTALL(/etc/passwd, 10);
  #comment* 
  #comment*   SAVEINSTALL(/etc/passwd, 10, compare);
  #comment* 
  #comment*   SAVEINSTALL(/etc/passwd, 10, compare\,nochkgroup);
  #comment*
  #comment* PURPOSE
  #comment* The following macro installs the file(s) to it's first argument
  #comment* and keeps it's second argument's number of backups. If the
  #comment* optional third argument is provided this single option is
  #comment* passed as an option to the rdist install directive. Note
  #comment* that commas in the install_args must be escaped with a \. So
  #comment* install_args can be "compare\,nochkgrp" (without the
  #comment* quotes). This will result in an install command of:
  #comment* 
  #comment*    install -osavetargets,compare,nochkgrp
  #comment* 
  #comment* The SAVEINSTALL macro is the preferred way to push/backup
  #comment* individual files/directories. Use BACKUP for just the
  #comment* backup portion if you have more complex needs.
  #comment* EXAMPLE
  #comment* The following example will install /etc/passwd and keep 7 backups.
  #comment*   label-manual:
  #comment*   $C/passwd/tigris -> ( tigris ) 
  #comment*    		SAVEINSTALL(/etc/passwd, 7);
  #comment*
  #comment* It generates something like:
  #comment*   label-manual:
  #comment*   $C/passwd/tigris -> ( tigris ) 
  #comment*    	install -osavetargets /etc/passwd;
  #comment*    	special "if [ -r ${REMFILE}.OLD ]; then mv --backup=numbered ${REMFILE}.OLD ${REMFILE}.SAVED; ... xargs -r rm -f ; fi";
  #comment*
  #comment* The following example will install /etc/passwd and keep 7
  #comment* backups, but only push the file if the contents compare
  #comment* differently.
  #comment*   label-manual:
  #comment*   $C/passwd/tigris -> ( tigris ) 
  #comment*    		SAVEINSTALL(/etc/passwd, 7, compare);
  #comment*
  #comment* The generated output is identical to the above except for
  #comment* the install command which looks like:
  #comment*    	install -osavetargets,compare /etc/passwd;
  #comment*
  #comment* The following example will install (if needed) the files in the
  #comment* directory /etc/nagios and keep 3 backups for each file:
  #comment*   label-manual:
  #comment*   $C/nagios/{hosts.cfg,services,cfg,time.cfg}  -> ( tigris ) 
  #comment*    		SAVEINSTALL(/etc/nagios/., 3);
  #comment* SEE ALSO
  #comment* BACKUP
  #comment******

#bigdef  SAVEINSTALL(TARGET, MAX_VERSIONS, ARGS...)
install -osavetargets,##ARGS TARGET;
	special "PATH=/etc/config/bin:$PATH; export PATH; if [ -r ${REMFILE}.OLD ]; then mv --backup=numbered ${REMFILE}.OLD ${REMFILE}.SAVED; ls ${REMFILE}.SAVED.*~ 2>/dev/null | sort -t '~' -k 2,2nr | sed '1,##MAX_VERSIONS##d' | xargs -r rm -f ; fi RDIST_SPECIAL_FILTER"##

#endbigdef

  #comment****m* rdist_macros/VERIFYFILE
  #comment* NAME
  #comment*   VERIFYFILE - Diff an installed file against the one in the repository.
  #comment* SYNOPSIS
  #comment*   VERIFYFILE(/etc/ntp.conf);
  #comment* PURPOSE
  #comment* The following macro is given the name of the file (on the
  #comment* client) that it should compare against. Then it installs a
  #comment* copy of the repository file on the remote system and
  #comment* does a unidiff between the installed and repository
  #comment* copies. It returns the unidiff output.
  #comment*
  #comment* It is useful for checking a file before it is pushed or
  #comment* verifying the correct file is installed.
  #comment* EXAMPLE
  #comment* A sample rule is:
  #comment*    ntp-verify:
  #comment*    $C/ntp/clients.conf -> ${NTP_SLAVES}
  #comment* 	   VERIFYFILE(/etc/ntp.conf);
  #comment******

#bigdef  VERIFYFILE(TARGET)
install TARGET.verify ;
	special "diff -u TARGET ${REMFILE}; exit=$?; rm -f ${REMFILE}; exit $exit RDIST_SPECIAL_FILTER"##
#endbigdef

  #comment standard header stanza for the Distfile.
__NEWLINE__
# file prepared from __BASE_FILE__ on __DATE__ at __TIME__
# using filepp version __VERSION__
__NEWLINE__
